/* array.h
 *
 * Copyright 2018 Bruce Cowan <bruce@bcowan.me.uk>
 *
 * This file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * This file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#pragma once

#include <stdbool.h>
#include <stddef.h>

typedef struct _Array Array;

typedef void   (*FreeFunc)    (void *element);
typedef double (*ValueFunc)   (const void *element);
typedef int    (*CompareFunc) (const void *a, const void *b);

// Fundamental functions
Array * array_new         (FreeFunc    func);
Array * array_ref         (Array      *array);
void    array_unref       (Array      *array);

void    array_add         (Array      *array,
                           const void *val);
bool    array_remove_fast (Array      *array,
                           size_t      index);
bool    array_remove      (Array      *array,
                           size_t      index);
bool    array_set         (Array      *array,
                           size_t      index,
                           const void *val);

// Getters    
const void * array_get          (const Array *array,
                                 size_t       index);
size_t       array_get_length   (const Array *array);
size_t       array_get_capacity (const Array *array);

// Misc
void array_sort (Array      *array,
                 CompareFunc func);
